@@ -28,11 +28,12 @@ use warnings;
 use autodie;
 
 use Dpkg::Version qw(version_check);
-use List::MoreUtils qw(any true);
+use List::MoreUtils qw(any true uniq);
 
 use Lintian::Architecture qw(:all);
 use Lintian::Data ();
 use Lintian::Check qw(check_maintainer);
+use Lintian::Command qw(safe_qx);
 use Lintian::Relation qw(:constants);
 use Lintian::Relation::Version qw(versions_compare);
 use Lintian::Tags qw(tag);
@@ -52,10 +53,17 @@ our $known_build_essential
 our $KNOWN_BINARY_FIELDS = Lintian::Data->new('fields/binary-fields');
 our $KNOWN_UDEB_FIELDS = Lintian::Data->new('fields/udeb-fields');
 our $KNOWN_BUILD_PROFILES = Lintian::Data->new('fields/build-profiles');
-our $KNOWN_VCS_BROWSERS
-  = Lintian::Data->new('fields/vcs-browsers', qr/\s*~~\s*/, sub { $_[1]; });
+our $KNOWN_VCS_HOSTERS= Lintian::Data->new(
+    'fields/vcs-hosters',
+    qr/\s*~~\s*/,
+    sub {
+        my @ret = split(',', $_[1]);
+        return \@ret;
+    });
 my $KNOWN_INSECURE_HOMEPAGE_URIS
   = Lintian::Data->new('fields/insecure-homepage-uris');
+my $DERIVATIVE_VERSIONS= Lintian::Data->new('fields/derivative-versions',
+    qr/\s*~~\s*/, sub { $_[1]; });
 
 our %KNOWN_ARCHIVE_PARTS = map { $_ => 1 } ('non-free', 'contrib');
 
@@ -129,8 +137,9 @@ my %VCS_EXTRACT = (
     cvs     => sub { return shift =~ /^(.+?)(?:\s+(\S*))?$/;},
     darcs   => sub { return @_;},
     hg      => sub { return @_;},
-    # git uri followed by optional " -b " + branchname:
-    git     => sub { return shift =~ /^(.+?)(?:\s+-b\s+(\S*))?$/;},
+    # git uri followed by optional "[subdir]", "-b branchname" etc.
+    git     =>
+      sub { return shift =~ /^(.+?)(?:\s+\[(\S*)\])?(?:\s+-b\s+(\S*))?$/;},
     svn     => sub { return @_;},
     # New "mtn://host?branch" uri or deprecated "host branch".
     mtn     => sub { return shift =~ /^(.+?)(?:\s+\S+)?$/;},
@@ -159,7 +168,7 @@ my %VCS_CANONIFY = (
         if (
             $_[0] =~ s{\@(?:cvs\.alioth|anonscm)\.debian\.org:/cvsroot/}
                       {\@anonscm.debian.org:/cvs/}
-          ) {
+        ) {
             $_[1] = 'vcs-field-bitrotted';
         }
         $_[0]=~ s{\@\Qcvs.alioth.debian.org:/cvs/}{\@anonscm.debian.org:/cvs/};
@@ -178,7 +187,7 @@ my %VCS_CANONIFY = (
         if (
             $_[0] =~ s{git://(?:git|anonscm)\.debian\.org/~}
                       {https://anonscm.debian.org/git/users/}
-          ) {
+        ) {
             $_[1] = 'vcs-git-uses-invalid-user-uri';
         }
         $_[0] =~ s{https?\Q://git.debian.org/\E(?:git/?)?}
@@ -303,7 +312,7 @@ sub run {
                     && $debian =~ m/\A
                             (?:[^.]+ubuntu[^.]+)(?:\.\d+){1,3}(\..*)?
                     \Z/oxsm
-                  ) {
+                ) {
                     $ubuntu = 1;
                     $extra = $1;
                 }
@@ -325,7 +334,7 @@ sub run {
                     (rc|alpha|beta|pre(?:view|release)?)
                    ([^a-z].*|\Z)
                  /xsm
-              ) {
+            ) {
                 my $expected = $upstream;
                 my $rc = $1;
                 my $rest = $2//'';
@@ -352,6 +361,15 @@ sub run {
                 }
             }
 
+            if (!$info->native) {
+                foreach my $re ($DERIVATIVE_VERSIONS->all) {
+                    next if $version =~ m/$re/;
+                    my $explanation = $DERIVATIVE_VERSIONS->value($re);
+                    tag 'invalid-version-number-for-derivative', $version,
+                      "($explanation)";
+                }
+            }
+
             my $name = $info->field('package');
             if (   $name
                 && $PERL_CORE_PROVIDES->known($name)
@@ -383,11 +401,21 @@ sub run {
 
     if ($type eq 'source') {
         for my $bin ($info->binaries) {
-            my $arch = $info->binary_field($bin, 'architecture');
-            my $fname = "debian/$bin.lintian-overrides.$arch";
             next unless $info->binary_field($bin, 'multi-arch', '') eq 'same';
-            tag 'multi-arch-same-package-has-arch-specific-overrides', $fname
-              if $info->index_resolved_path($fname);
+            my $wildcard = $info->binary_field($bin, 'architecture');
+            my @arches = split(
+                ' ',
+                safe_qx(
+                    'dpkg-architecture', '--match-wildcard',
+                    $wildcard, '--list-known'
+                ));
+            push(@arches, $wildcard); # original wildcard should be included
+            foreach my $arch (uniq @arches) {
+                my $fname = "debian/$bin.lintian-overrides.$arch";
+                tag 'multi-arch-same-package-has-arch-specific-overrides',
+                  $fname
+                  if $info->index_resolved_path($fname);
+            }
         }
     }
 
@@ -438,6 +466,15 @@ sub run {
                 if ($pkg =~ /-docs?$/ && $archs[0] ne 'all') {
                     tag 'documentation-package-not-architecture-independent';
                 }
+                if ($pkg =~ /^r-(?:cran|bioc|other)-/ and $archs[0] ne 'all') {
+                    for my $file ($info->sorted_index) {
+                        next if $file->is_dir;
+                        next if $file !~ m,^usr/lib/R/.*/DESCRIPTION,;
+                        tag 'r-package-not-arch-all'
+                          if $file->file_contents =~ m/NeedsCompilation: no/m;
+                        last;
+                    }
+                }
             }
         }
         # Used for later tests.
@@ -527,7 +564,7 @@ sub run {
             \s*
             # Optional Version e.g. (1.0)
             (?:\((?:\d+:)?(?:[-\.+:a-zA-Z0-9~]+?)(?:-[\.+a-zA-Z0-9~]+)?\))?\s*$/x
-          ) {
+        ) {
             tag 'source-field-malformed', $source;
         }
     }
@@ -676,10 +713,8 @@ sub run {
         if ($homepage=~ m,bioconductor\.org/packages/.*/bioc/html/.*\.html*$,){
             tag 'homepage-for-bioconductor-package-not-canonical', $orig;
         }
-        foreach my $re ($KNOWN_INSECURE_HOMEPAGE_URIS->all) {
-            next unless $homepage =~ m/$re/;
+        if ($KNOWN_INSECURE_HOMEPAGE_URIS->matches_any($homepage)) {
             tag 'homepage-field-uses-insecure-uri', $orig;
-            last;
         }
     } elsif (not $info->native) {
         if ($type eq 'source') {
@@ -708,27 +743,6 @@ sub run {
 
     #---- Package relations (binary package)
 
-    # Check whether the package looks like a metapackage, used for later
-    # dependency checks.  We consider a package to possibly be a metapackage if
-    # it is a binary package with no files outside of /usr/share/doc and a few
-    # other directories found in metapackages.  This also catches documentation
-    # packages, but that doesn't matter for our purposes.
-    my $metapackage = 0;
-    if ($type eq 'binary') {
-        $metapackage = 1;
-        for my $file ($info->sorted_index) {
-            next if $file->is_dir;
-            next if $file =~ m%^usr/share/doc/%;
-            next if $file =~ m%^usr/share/lintian/overrides/%;
-            next if $file =~ m%^usr/share/cdd/%;
-            $metapackage = 0;
-            last;
-        }
-
-        # Packages we say are metapackages are always metapackages even if
-        # they don't look like it.
-        $metapackage = 1 if $KNOWN_METAPACKAGES->known($pkg);
-    }
     if (($type eq 'binary') || ($type eq 'udeb')) {
         my $javalib = 0;
         my $replaces = $info->relation('replaces');
@@ -736,7 +750,7 @@ sub run {
         $javalib = 1 if($pkg =~ m/^lib.*-java$/o);
         for my $field (
             qw(depends pre-depends recommends suggests conflicts provides enhances replaces breaks)
-          ) {
+        ) {
             next unless defined $info->field($field);
             #Get data and clean it
             my $data = $info->field($field);
@@ -837,7 +851,8 @@ sub run {
 
                     tag 'depends-on-metapackage', "$field: $part_d_orig"
                       if (  $KNOWN_METAPACKAGES->known($d_pkg)
-                        and not $metapackage
+                        and not $KNOWN_METAPACKAGES->known($pkg)
+                        and not $info->is_pkg_class('any-meta')
                         and &$is_dep_field($field));
 
                     # diffutils is a special case since diff was
@@ -923,6 +938,7 @@ sub run {
                       and &$is_dep_field($field)
                       and not $pkg =~ m/^dh-/
                       and not $pkg =~ m/-(source|src)$/
+                      and not $info->is_pkg_class('any-meta')
                       and not $DH_ADDONS_VALUES{$pkg};
 
                     # default-jdk-doc must depend on openjdk-X-doc (or
@@ -1050,7 +1066,7 @@ sub run {
         my %depend;
         for my $field (
             qw(build-depends build-depends-indep build-depends-arch build-conflicts build-conflicts-indep build-conflicts-arch)
-          ) {
+        ) {
             if (defined $info->field($field)) {
                 #Get data and clean it
                 my $data = $info->field($field);
@@ -1210,7 +1226,7 @@ sub run {
             $depend{'build-conflicts'},
             $depend{'build-conflicts-indep'},
             $depend{'build-conflicts-arch'}
-          ) {
+        ) {
             next unless $_;
             for my $conflict (split /\s*,\s*/, $_) {
                 if ($build_all->implies($conflict)) {
@@ -1251,22 +1267,20 @@ sub run {
             tag 'build-depends-on-python-dev-with-no-arch-any';
         }
 
+        my $bdepends = $info->relation('build-depends');
+
         # libmodule-build-perl
         # matches() instead of implies() because of possible OR relation
-        if ($info->relation('build-depends-indep')
-            ->matches(qr/^libmodule-build-perl$/, VISIT_PRED_NAME)
-            && !$info->relation('build-depends')
-            ->matches(qr/^libmodule-build-perl$/, VISIT_PRED_NAME)) {
-            tag 'libmodule-build-perl-needs-to-be-in-build-depends';
-        }
+        tag 'libmodule-build-perl-needs-to-be-in-build-depends'
+          if $info->relation('build-depends-indep')
+          ->matches(qr/^libmodule-build-perl$/, VISIT_PRED_NAME)
+          && !$bdepends->matches(qr/^libmodule-build-perl$/,VISIT_PRED_NAME);
 
         # libmodule-build-tiny-perl
-        if ($info->relation('build-depends-indep')
-            ->implies('libmodule-build-tiny-perl')
-            && !$info->relation('build-depends')
-            ->implies('libmodule-build-tiny-perl')) {
-            tag 'libmodule-build-tiny-perl-needs-to-be-in-build-depends';
-        }
+        tag 'libmodule-build-tiny-perl-needs-to-be-in-build-depends'
+          if $info->relation('build-depends-indep')
+          ->implies('libmodule-build-tiny-perl')
+          && !$bdepends->implies('libmodule-build-tiny-perl');
     }
 
     #----- Origin
@@ -1285,8 +1299,7 @@ sub run {
         tag 'redundant-bugs-field'
           if $bugs =~ m,^debbugs://bugs.debian.org/?$,i;
 
-        tag 'bugs-field-does-not-refer-to-debian-infrastructure', $bugs,
-          "(line $.)"
+        tag 'bugs-field-does-not-refer-to-debian-infrastructure', $bugs
           unless $bugs =~ m,\.debian\.org, or $pkg =~ /[-]dbgsym$/;
     }
 
@@ -1347,9 +1360,21 @@ sub run {
                   if $uri =~ m%rev=0&sc=0%;
             } else {
                 $seen_vcs{$vcs}++;
+                foreach my $regex ($KNOWN_VCS_HOSTERS->all) {
+                    foreach my $re_vcs (@{$KNOWN_VCS_HOSTERS->value($regex)}) {
+                        if (   $uri =~ m/^($regex.*)/xi
+                            && $vcs ne $re_vcs
+                            && $vcs ne 'browser') {
+                            tag 'vcs-field-mismatch',
+                              "vcs-$vcs != vcs-$re_vcs",
+                              $uri;
+                            last;
+                        }
+                    }
+                }
             }
             if ($uri =~ m{//(.+)\.debian\.org/}) {
-                tag 'vcs-deprecated-in-debian-infrastructure', "vcs-$vcs", $uri
+                tag 'vcs-obsolete-in-debian-infrastructure', "vcs-$vcs", $uri
                   unless $1 =~ m{^(?:salsa|.*\.dgit)$};
             } else {
                 tag 'orphaned-package-not-maintained-in-debian-infrastructure',
@@ -1370,8 +1395,8 @@ sub run {
 
     # Check for missing Vcs-Browser headers
     if (!defined $info->field('vcs-browser')) {
-        foreach my $regex ($KNOWN_VCS_BROWSERS->all) {
-            my $vcs = $KNOWN_VCS_BROWSERS->value($regex);
+        foreach my $regex ($KNOWN_VCS_HOSTERS->all) {
+            my $vcs = @{$KNOWN_VCS_HOSTERS->value($regex)}[0];
             if ($info->field("vcs-$vcs", '') =~ m/^($regex.*)/xi) {
                 tag 'missing-vcs-browser-field', "vcs-$vcs", $1;
                 last; # Only warn once
