@@ -23,10 +23,12 @@ use strict;
 use warnings;
 use autodie;
 
+use List::MoreUtils qw(none);
+
 use Lintian::Data;
 use Lintian::Tags qw(tag);
 use Lintian::Util qw(drain_pipe internal_error is_string_utf8_encoded open_gz
-  signal_number2name strip normalize_pkg_path);
+  signal_number2name strip normalize_pkg_path slurp_entire_file);
 use Lintian::SlidingWindow;
 
 use constant BLOCKSIZE => 16_384;
@@ -295,9 +297,9 @@ my $OBSOLETE_PATHS = Lintian::Data->new(
     });
 
 sub run {
-    my ($pkg, $type, $info, $proc) = @_;
+    my ($pkg, $type, $info, $proc, $group) = @_;
     my ($is_python, $is_perl, $has_binary_perl_file, $has_public_executable,
-        $has_public_shared_library);
+        $has_public_shared_library, $build_path);
     my @nonbinary_perl_files_in_lib;
     my %linked_against_libvga;
     my @devhelp;
@@ -316,6 +318,11 @@ sub run {
     my $multiarch_dir = $MULTIARCH_DIRS->value($arch);
     my $ppkg = quotemeta($pkg);
 
+    my $buildinfo = $group->get_buildinfo_processable;
+    if ($buildinfo) {
+        $build_path = $buildinfo->info->field('build-path', '');
+    }
+
     my %header_dirs = ('usr/include/' => 1);
     foreach my $x ($MULTIARCH_DIRS->all) {
         my $dir = $MULTIARCH_DIRS->value($x);
@@ -343,8 +350,6 @@ sub run {
     # Check if package is empty
     my $is_dummy = $info->is_pkg_class('any-meta');
 
-    my $has_sensible_utils = $info->relation('all')->implies('sensible-utils');
-
     # read data from objdump-info file
     foreach my $file (sort keys %{$info->objdump_info}) {
         my $objdump = $info->objdump_info->{$file};
@@ -416,8 +421,13 @@ sub run {
         my $owner = $file->owner . '/' . $file->group;
         my $operm = $file->operm;
         my $link = $file->link;
+        my $finfo = $file->file_info;
 
-        $arch_dep_files = 1 if $fname !~ m,^usr/share/,o && $fname ne 'usr/';
+        $arch_dep_files = 1
+          if not $file->is_dir
+          and $fname !~ m,^usr/share/,o
+          and $finfo
+          and $finfo !~ m/\bASCII text\b/;
 
         if (exists($PATH_DIRECTORIES{$file->dirname})) {
             $has_public_executable = 1;
@@ -920,7 +930,7 @@ sub run {
                     and $fname !~ m,^usr/bin/(?:X11|mh)/,) {
                     tag 'subdir-in-usr-bin', $file;
                 }
-                # check old style config script
+                # check old style config scripts
                 elsif ( $file->is_regular_file
                     and $fname =~ m,-config$,
                     and $script{$file}) {
@@ -1494,12 +1504,32 @@ sub run {
                 tag 'nfs-temporary-file-in-package', $file;
             }
 
-            # ---------------- using sensible-utils w/o dependency
-            tag 'missing-depends-on-sensible-utils', $file
-              if not $has_sensible_utils
-              and $fname !~ m,^usr/share/(?:doc|locale)/,
-              and $source_pkg ne 'lintian'
-              and detect_sensible_utils($file);
+            # ---------------- contents checks
+            my %checks
+              = get_checks_for_file($info, $file, $source_pkg, $build_path);
+
+            if (%checks) {
+                my $strings = slurp_entire_file($info->strings($file));
+
+                foreach my $tag (sort keys %checks) {
+                    my $regex = $checks{$tag};
+
+                    # If we have strings(1) output (eg. we are an ELF
+                    # binary) then prefer that.
+                    if ($strings) {
+                        tag $tag, $file if $strings =~ m,^\Q$regex\E,m;
+                    } else {
+                        my $fd = $file->open(':raw');
+                        my $sfd = Lintian::SlidingWindow->new($fd);
+                        while (my $block = $sfd->readwindow) {
+                            next unless $block =~ $regex;
+                            tag $tag, $file;
+                            last;
+                        }
+                        close($fd);
+                    }
+                }
+            }
 
             # ---------------- documentation files
             unless($fname =~ m,^etc/, or $fname =~ m,^usr/share/doc/,) {
@@ -2005,8 +2035,15 @@ sub run {
         my $file = $info->control_index_resolved_path($1);
         next if not $file or not $file->is_open_ok;
 
-        tag 'missing-depends-on-sensible-utils', $file
-          if not $has_sensible_utils and detect_sensible_utils($file);
+        my %checks
+          = get_checks_for_file($info, $file, $pkg_section, $build_path);
+        my $fd2 = $file->open;
+        while (<$fd2>) {
+            foreach my $tag (sort keys %checks) {
+                tag $tag, $file, "(line $.)" if $_ =~ $checks{$tag};
+            }
+        }
+        close($fd2);
     }
     close($fd);
 
@@ -2309,18 +2346,25 @@ sub detect_privacy_breach {
     return;
 }
 
-sub detect_sensible_utils {
-    my ($file) = @_;
+sub get_checks_for_file {
+    my ($info, $file, $source_pkg, $build_path) = @_;
+    my %checks;
 
-    my $fd = $file->open(':raw');
-    my $sfd = Lintian::SlidingWindow->new($fd, sub { $_= lc($_); }, BLOCKSIZE);
+    return %checks if $source_pkg eq 'lintian';
 
-    while (my $block = $sfd->readwindow) {
-        return 1
-          if $block=~ m/(?:select-editor|sensible-(?:browser|editor|pager))\b/;
-    }
-    close($fd);
-    return;
+    $checks{'missing-depends-on-sensible-utils'}
+      = '(?:select-editor|sensible-(?:browser|editor|pager))\b'
+      if $file !~ m,^usr/share/(?:doc|locale)/,
+      and not $info->relation('all')->implies('sensible-utils');
+
+    $checks{'uses-dpkg-database-directly'} = '/var/lib/dpkg'
+      if $file !~ m,^usr/share/(?:doc|locale)/,
+      and $info->field('section', '') ne 'debian-installer'
+      and none { $_ eq $source_pkg } qw(base-files dpkg lintian);
+
+    $checks{'file-references-package-build-path'} = $build_path if $build_path;
+
+    return %checks;
 }
 
 1;
