@@ -39,6 +39,9 @@ use Lintian::Data;
 # Init script that do not need a service file
 my $INIT_WHITELIST = Lintian::Data->new('systemd/init-whitelist');
 
+# Known security flags
+my $HARDENING_FLAGS = Lintian::Data->new('systemd/hardening-flags');
+
 # Usual WantedBy= targets
 my $WANTEDBY_WHITELIST = Lintian::Data->new('systemd/wantedby-whitelist');
 
@@ -220,8 +223,13 @@ sub check_systemd_service_file {
     }
 
     if (not $file->is_symlink or $file->link ne '/dev/null') {
+        #<<< no perltidy
         my @wanted_by
-          = extract_service_file_values($file, 'Install', 'WantedBy',1);
+          = extract_service_file_values($file, 'Install', 'WantedBy', 1);
+        my $is_oneshot =
+           any { /^oneshot$/ }
+           extract_service_file_values($file, 'Service', 'Type', 1);
+        #>>>
         foreach my $target (@wanted_by) {
             tag 'systemd-service-file-refers-to-unusual-wantedby-target',
               $file, $target
@@ -230,19 +238,25 @@ sub check_systemd_service_file {
         }
         tag 'systemd-service-file-missing-documentation-key', $file,
           unless extract_service_file_values($file, 'Unit', 'Documentation',1);
-        #<<< no perltidy - These are better formatted one per line
         tag 'systemd-service-file-missing-install-key', $file,
-          unless @wanted_by
+             unless @wanted_by
           or extract_service_file_values($file, 'Install', 'RequiredBy',1)
           or extract_service_file_values($file, 'Install', 'Also',1)
-          or any { /^oneshot$/ } extract_service_file_values($file, 'Service', 'Type')
+          or $is_oneshot
           or $file =~ m,@\.service$,;
-        #>>>
         my @pidfile = extract_service_file_values($file,'Service','PIDFile',1);
         foreach my $x (@pidfile) {
             tag 'systemd-service-file-pidfile-refers-to-var-run', $file, $x
               if $x =~ m,^/var/run/,;
         }
+        my $seen_hardening;
+        foreach my $x ($HARDENING_FLAGS->all) {
+            next unless extract_service_file_values($file, 'Service', $x, 1);
+            $seen_hardening = 1;
+            last;
+        }
+        tag 'systemd-service-file-missing-hardening-features', $file
+          unless $seen_hardening or $is_oneshot;
     }
 
     return 1;
