@@ -82,7 +82,7 @@ my @ENV_VARS = (
 ### "Normal" application variables
 my %conf_opt;                   #names of options set in the cfg file
 my %opt = (                     #hash of some flags from cmd or cfg
-    # Init some cmd-line value defaults
+     # Init some cmd-line value defaults
     'debug'             => 0,
 );
 
@@ -194,7 +194,7 @@ EOT-EOT-EOT
         print <<"EOT-EOT-EOT";
     --tag-display-limit X     Specify "tag per package" display limit
     --no-tag-display-limit    Disable "tag per package" display limit
-                              (equivalant to --tag-display-limit=0)
+                              (equivalent to --tag-display-limit=0)
 EOT-EOT-EOT
     }
 
@@ -674,7 +674,7 @@ sub main {
     if (
         not(   ($action eq 'unpack')
             or ($action eq 'check'))
-      ) {
+    ) {
         fatal_error("invalid action $action specified");
     }
 
@@ -822,7 +822,8 @@ sub main {
         my $warnings = $overrides{warnings} || 0;
         my $info = $overrides{info} || 0;
         my $total = $errors + $warnings + $info;
-        if ($total > 0) {
+        my $unused = $TAGS->{unused_overrides};
+        if ($total > 0 or $unused > 0) {
             my $text
               = ($total == 1)
               ? "$total tag overridden"
@@ -841,7 +842,15 @@ sub main {
             if ($info) {
                 push(@output, "$info info");
             }
-            msg("$text (". join(', ', @output). ')');
+            if (@output) {
+                $text .= ' (' . join(', ', @output). ')';
+            }
+            if ($unused == 1) {
+                $text .= "; $unused unused override";
+            } elsif ($unused > 1) {
+                $text .= "; $unused unused overrides";
+            }
+            msg($text);
         }
     }
 
